#!/bin/bash
##################################
# HA-Lizard version 1.7.2
##################################
#################################################################################################
#
# HA-Lizard - Open Source High Availability Framework for Xen Cloud Platform and XenServer
#
# Copyright 2013 Salvatore Costantino
# ha@pulsesupply.com
#
# This file is part of HA-Lizard.
#
#    HA-Lizard is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    HA-Lizard is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with HA-Lizard.  If not, see <http://www.gnu.org/licenses/>.
#
##################################################################################################
exec 1> >(logger -t ha-lizard-NOTICE-$0)
exec 2> >(logger -t ha-lizard-NOTICE-$0)

if [ -s /etc/ha-lizard/ha-lizard.pool.conf ]
then
	source /etc/ha-lizard/ha-lizard.pool.conf	#global configuration parameters for all hosts in pool - dynamic for pool
else
	cat /etc/ha-lizard/scripts/install.params > /etc/ha-lizard/ha-lizard.pool.conf
fi

source /etc/ha-lizard/ha-lizard.conf		#override configuration settings for this host - static for this host
source /etc/ha-lizard/ha-lizard.func

if [ -d $MAIL_SPOOL ]
then
        log "Mail Spool Directory Found $MAIL_SPOOL"
else
        `mkdir $MAIL_SPOOL`
        if [ $? = 0 ]
        then
                log "Successfully created mail spool directory $MAIL_SPOOL"
        else
                log "Failed to create mail spool - not suppressing duplicate notices"
        fi
fi

if [ ! -f $MAIL_SPOOL/count ]
then
        touch $MAIL_SPOOL/count
        echo 0 > $MAIL_SPOOL/count
fi

CURRENT_COUNT=`cat $MAIL_SPOOL/count`

if [ $CURRENT_COUNT -gt 10000 ]
then
        log "Resetting iteration counter"
        echo 1 > $MAIL_SPOOL/count
        CURRENT_COUNT=0
fi

NEW_COUNT=$(($CURRENT_COUNT + 1))
log "This iteration is count $NEW_COUNT"
echo $NEW_COUNT > $MAIL_SPOOL/count

if [ -e /etc/xensource/pool.conf ]
then
	log "Checking if this host is a Pool Master or Slave"
	STATE=`/bin/cat /etc/xensource/pool.conf`
	log "This host's pool status = $STATE"
else
	log "/etc/xensource/pool.conf missing. Cannot determine master/slave status."
	email "/etc/xensource/pool.conf missing. Cannot determine master/slave status."
	exit 1
fi


if [ $STATE = "master" ]
then
	log "Checking if ha-lizard is enabled for this pool"
	check_ha_enabled
	case $? in
        	0)
                	log "ha-lizard is enabled"
                	;;
        	1)      log "ha-lizard is disabled"
                	exit 1
                	;;
        	2)      log "ha-lizard-enabled state unknown - exiting"
                	exit 2
                	;;
		*)	log "check_ha_enabled returned error: $? - exiting"
			exit $?
			;;
	esac
fi

$TIMEOUT 10 $CLI_TOOL get > "$GLOBAL_CONF_TMP"
if [ $? != "124" ]
then
	$CAT $GLOBAL_CONF_TMP > "$GLOBAL_CONF"
	log "Successfully updated global pool configuration settings in $GLOBAL_CONF. Settings take effect on subsequent run"
	log "`$CAT $GLOBAL_CONF`"
else
	log "Failed to update global pool configuration settings in $GLOBAL_CONF - Check Configuration!"
	email "Failed to update global pool configuration settings in $GLOBAL_CONF - Check Configuration!"
fi


if [ $STATE = "master" ]
then
	log "This host detected as pool  Master"	
	
	NUM_HOSTS=`$XE host-list | grep "uuid ( RO)"| wc -l`
	if [ $? = 0 ]
	then
		log "Found $NUM_HOSTS hosts in pool"
	else
		log "Failed to find total number of hosts in pool"
	fi

	log "Calling function write_pool_state"
	write_pool_state &
	log "Calling function autoselect_slave"
	autoselect_slave &
	log "Calling function check_slave_status"
	check_slave_status
	
	case "$?" in 
		2)	
			log "Function check_slave_status Host Power = Off, calling vm_mon"
			vm_mon
			;;
		1)
			log "Function check_slave_status failed to fence failed host.. checking whether to attempt starting failed VMs"
			log "FENCE_HA_ONFAIL is set to: $FENCE_HA_ONFAIL"
			if [ $FENCE_HA_ONFAIL = "1" ]
			then
				log "FENCE_HA_ONFAIL is set to: $FENCE_HA_ONFAIL, calling vm_mon"
				vm_mon
			else
				log "FENCE_HA_ONFAIL is set to: $FENCE_HA_ONFAIL, not attempting to start VMs"	
			fi
			;;
		0)
			log "Function check_slave_status reported no failures: calling vm_mon"
			vm_mon
			;;
		*)
			log "Calling function vm_mon"
			vm_mon
			$ECHO "0" > $STATE_PATH/rebooted
			;;
	esac
fi

if [[ $STATE == slave* ]]
then
	master_ip $STATE
	
	if [ -e $STATE_PATH/autopromote_uuid ]
	then
		THIS_SLAVE_UUID=`$CAT $STATE_PATH/local_host_uuid`
		AUTOPROMOTE_UUID=`$CAT $STATE_PATH/autopromote_uuid`
		if [ $THIS_SLAVE_UUID = $AUTOPROMOTE_UUID ]
		then
			log "This slave - $HOST: $THIS_SLAVE_UUID selected as allowed to become master: setting ALLOW_PROMOTE_MASTER=1"
			ALLOW_PROMOTE_SLAVE=1
		else
			log "This slave- $HOST: $THIS_SLAVE_UUID not permitted to become master"
		fi
		
	else
		log "Missing file - $STATE_PATH/autopromote_uuid - cannot validate autopromote_status"
		email "Missing file - $STATE_PATH/autopromote_uuid - cannot validate autopromote_status"
		THIS_SLAVE_UUID=
	fi
	

	if check_xapi $MASTER_IP -a $POLL_MASTER
	then

		log "Pool Master is OK - calling function check_ha_enabled - updating local status"
		check_ha_enabled
        	if [ $? = "0" ]
		then
        		`$ECHO true > $STATE_PATH/ha_lizard_enabled`
		else
        		`$ECHO false > $STATE_PATH/ha_lizard_enabled`
		fi

		log "Checking state file for status if ha-lizard is enabled"
        	if [ -e "$STATE_PATH/ha_lizard_enabled" ]
        	then
                	log "Statefile $STATE_PATH/ha_lizard_enabled found: checking if ha-lizard is enabled"
                	ha_lizard_STAT=`$CAT $STATE_PATH/ha_lizard_enabled`
                	if [ $ha_lizard_STAT = "true" ]
                	then
                        	log "ha-lizard is enabled - continuing"
                	else
                        	log "ha-lizard is disabled - exiting"
                        	exit 0
                	fi
        	fi
		
                log "Calling Function write_pool_state - updating local state files"
                write_pool_state &
		
		if [ $SLAVE_VM_STAT -eq "1" ]
		then
			log "Calling Function vm_mon - check if any VMs need to be started"
			vm_mon
		fi
	else
		log "Pool Master NOT OK - Checking if ha-lizard is enabled in latest state file"
                log "Checking if ha-lizard is enabled"
                if [ -e "$STATE_PATH/ha_lizard_enabled" ]
                then
                        log "Statefile $STATE_PATH/ha_lizard_enabled found: checking if ha-lizard is enabled"
                        ha_lizard_STAT=`$CAT $STATE_PATH/ha_lizard_enabled`
                        if [ $ha_lizard_STAT = "true" ]
                        then
                                log "ha-lizard is enabled - continuing"
                        else
                                log "ha-lizard is disabled - exiting"
                                exit 0
                        fi
                fi

		log "Pool Master Monitor = Failed"
		email "Server $HOSTNAME: Failed to contact pool master - manual intervention may be required"
		log "Retry Count set to $XAPI_COUNT. Retrying $XAPI_COUNT times in $XAPI_DELAY second intervals.."
		COUNT=0 #reset loop counter
                while [ $COUNT -lt $XAPI_COUNT ]
		do
			log "Attempt $COUNT: Checking Pool Master Status"
			COUNT=$((COUNT+1))
			sleep $XAPI_DELAY
			
			if check_xapi $MASTER_IP
			then
				log "Pool Master Communication Restored"
				break
			else
				if [ $COUNT = $XAPI_COUNT ]
				then
					if [ -e $STATE_PATH/pool_num_hosts ]
					then
						NUM_HOSTS=`$CAT $STATE_PATH/pool_num_hosts`
						log "Retrieving number of hosts in pool. Setting NUM_HOSTS = $NUM_HOSTS"
					else
						log "ERROR Retrieving number of hosts in pool. Setting NUM_HOSTS = UNKNOWN"
					fi
					
					log "Failed to reach Pool Master - Checking if this host promotes to Master.."
					
					if [[ $PROMOTE_SLAVE = "1" ]] && [[ $ALLOW_PROMOTE_SLAVE = "1" ]]	
					then
						log "Promote Slave enabled for this host. Calling promote_slave - attempt to become pool master"
						promote_slave
						
						if [ $? = "1" ]
						then
							log "Failed to promote slave - Pool master must be manually recovered!"	
							email "Failed to promote slave - Pool master must be manually recovered!"
							exit 1
						fi

						MASTER_UUID=`$CAT $STATE_PATH/master_uuid`
						log "State file MASTER UUID = $MASTER_UUID"

                                                if [ $FENCE_ENABLED = "1" ]
                                                then
                                                        `fence_host $MASTER_UUID stop`
							
							case $? in
								0)
									log "Master: $MASTER_UUID successfully fenced, attempting to start any failed VMs"
									;;
								1)
									log "Failed to fence Master: $MASTER_UUID. Checking whether FENCE_HA_ONFAIL is enabled"
									if [ $FENCE_HA_ONFAIL = "0" ]
									then
										log "FENCE_HA_ONFAIL is disabled, failed to fence Master, exiting - Major Error!, no pool master"
										email "FENCE_HA_ONFAIL is disabled, failed to fence Master, exiting - Major Error!, no pool master"
										exit 1	
									elif [ $FENCE_HA_ONFAIL = "1" ]
									then
										log "FENCE_HA_ONFAIL is enabled - attempted to recover any failed VMs"
									fi
									;;
							esac
						fi


						log "Retrieving list of VMs on failed master from local state file host.$MASTER_UUID.vmlist.uuid_array"
						FAILED_VMLIST=(`$CAT $STATE_PATH/host.$MASTER_UUID.vmlist.uuid_array`)

						for c in ${FAILED_VMLIST[@]}
						do
							log "Resetting Power State for VM: $c"
							$XE vm-reset-powerstate uuid=$c --force
							
							if [ $? = 0 ]
							then
								log "Power State for uuid: $c set to: halted"
							else
								log "Error resetting power state for VM UUID: $c"
							fi
						done
						

                                		RESET_IFS=$IFS
                                		IFS=","
                                		for v in `$XE pbd-list host-uuid=$MASTER_UUID --minimal`
                                		do
                                        		log "Resetting VDI: $v on host: $MASTER_UUID"
                                        		STORE=`$XE pbd-param-get uuid=$v param-name=sr-uuid`
                                        		$RESET_VDI $MASTER_UUID $STORE
                                        		if [ $? = "0" ]
                                        		then
                                                		log "Resetting VDI: $v Success!"
                                        		else
                                                		log "Resetting VDI: $v ERROR!"
                                        		fi
                                		done
						IFS=$RESET_IFS

						if [ $SLAVE_HA = 1 ]
						then
							log "Slave HA is ON, Master is unreachable  - Checking for VMs or appliances to start in this pool"
							sleep 5
							vm_mon
						else
							log "Slave HA is OFF, Master is unreachable  - Not Attempting Restore - Manual Intervention Needed"	
						fi
					else
						log "PROMOTE_SLAVE is disabled - Not Promoting this host - Manual Intervention Needed"
					fi
				fi
			fi
		done

	fi
fi
